﻿#include "s3e.h"
#include "Iw2D.h"
#include "IwGx.h"
#include "IwResManager.h"
#include "IwSound.h"

#include "IwGame.h"

//
//
//
// IwGame implementation
//
//
//
void CIwGame::Init()
{
	CurrentScene = NULL;
	NextScene = NULL;

	// Initialise Mamrlade graphics system and Iw2D module
	IwGxInit();
    Iw2DInit();

	// Initialise the input system
	CIwGameInput::Create();
	IW_GAME_INPUT->Init();

	// Init IwSound
	IwSoundInit();

	// Initialise the rsource manager
	IwResManagerInit();

#ifdef IW_BUILD_RESOURCES
	// Tell resource system how to convert WAV files
	IwGetResManager()->AddHandler(new CIwResHandlerWAV);
#endif

	// Set default background colour
	SetBackgroundColour(0, 0, 0, 0);

	// Get initial time stamp
	LastFrameTime = s3eTimerGetMs();
}

void CIwGame::SetBackgroundColour(uint8 r, uint8 g, uint8 b, uint8 a)
{
	// Set the default background clear colour
	IwGxSetColClear(r, g, b, a);
}

void CIwGame::Release()
{
	// Clean up scenes
	for (_Iterator it = Scenes.begin(); it != Scenes.end(); ++it)
		delete *it;
	Scenes.clear();

	// Shut down the input system
	IW_GAME_INPUT->Release();
	CIwGameInput::Destroy();

	// Shut down the resource manager
	IwResManagerTerminate();

	// Shutdown IwSound
	IwSoundTerminate();

	// Shut down Marmalade graphics system and the Iw2D module
	Iw2DTerminate();
	IwGxTerminate();
}

bool CIwGame::Update()
{
	// Calculate how long the last game frame took (capped at 60 and 10 fps) - We use this to scale all of our trasient variables that rely upon time so that everything moves at the same rate
	// regardless of our devices frame rate
	float dt = (float)(s3eTimerGetMs() - LastFrameTime) / 16.67f;
	if (dt < 1.0) dt = 1.0f;
	if (dt > 6.0f) dt = 6.0f;
	LastFrameTime = s3eTimerGetMs();

	// Clear the screen
	IwGxClear(IW_GX_COLOUR_BUFFER_F | IW_GX_DEPTH_BUFFER_F);

	// Update pointer system
	IW_GAME_INPUT->Update();

	// Update Iw Sound Manager
	IwGetSoundManager()->Update();

	// Check for scene change
	if (NextScene != CurrentScene)
	{
		// Notify scenes that there is a change of circumstances
		if (CurrentScene != NextScene)
		{
			if (CurrentScene != NULL)
			{
				CurrentScene->NotifyLostFocus(NextScene);
				if (CurrentScene->getAllowSuspend())
					CurrentScene->NotifySuspending(NextScene);
			}
			if (NextScene != NULL)
			{
				NextScene->NotifyGainedFocus(CurrentScene);
				if (NextScene->getAllowSuspend())
					NextScene->NotifyResuming(CurrentScene);
			}
			CurrentScene = NextScene;
		}
	}

	// Update all scenes that are not suspended
	for (_Iterator it = Scenes.begin(); it != Scenes.end(); ++it)
	{
		if (CurrentScene == *it)
		{
			(*it)->Update(dt);
		}
		else
		{
			if (!(*it)->getAllowSuspend())
			{
				(*it)->Update(dt);
			}
		}
	}

	return true;
}

void CIwGame::Draw()
{
	// draw all scenes that are not suspended
	for (_Iterator it = Scenes.begin(); it != Scenes.end(); ++it)
	{
		(*it)->Draw();
	}

	// Show the surface
	Iw2DSurfaceShow();

	// Yield to the operating system
	s3eDeviceYield(0);
}

void CIwGame::addScene(CIwGameScene *scene)
{
	Scenes.push_back(scene);
}
void CIwGame::removeScene(CIwGameScene* scene)
{
	removeScene(scene->getNameHash());
}
void CIwGame::removeScene(unsigned int name_hash)
{
	for (_Iterator it = Scenes.begin(); it != Scenes.end(); ++it)
	{
		if ((*it)->getNameHash() == name_hash)
		{
			delete *it;
			Scenes.erase(it);
			break;
		}
	}
}
CIwGameScene* CIwGame::findScene(unsigned int name_hash)
{
	for (_Iterator it = Scenes.begin(); it != Scenes.end(); ++it)
	{
		if ((*it)->getNameHash() == name_hash)
		{
			return *it;
		}
	}

	return NULL;
}
CIwGameScene* CIwGame::findScene(const char* name)
{
	return findScene(IwHashString(name));
}
CIwGameScene* CIwGame::getScene(int index)
{
	if (index < 0 || index >= Scenes.size())
		return NULL;

	return Scenes.element_at(index);
}
void CIwGame::clearScenes()
{
	for (_Iterator it = Scenes.begin(); it != Scenes.end(); ++it)
		delete *it;

	Scenes.clear();
}
void CIwGame::changeScene(CIwGameScene *new_scene)
{
	NextScene = new_scene;
}
bool CIwGame::changeScene(unsigned int name_hash)
{
	NextScene = findScene(name_hash);
	if (NextScene == NULL)
		return false;

	return true;
}
