#if !defined(_CIW_GAME_ANIMATION_H_)
#define _CIW_GAME_ANIMATION_H_

#include "IwGameUtil.h"
#include "IwGeom.h"
#include "IwList.h"

//
//
// CIwGameAnim
//
// 
class CIwGameAnim
{
	// Properties
protected:
	bool				Playing;					// Current pLaying state
	float				CurrentFrame;				// Current animation frame
	float				PlaybackSpeed;				// Rate at which to playback animation
	int					FrameCount;					// Total number of animations in the animation
	bool				FrameHasChanged;			// Set to true when a frame has changed
	bool				Looping;					// True if the animation loops
	int					NumLoops;					// Total number of loops played back so far
	int					MaxLoops;					// Maximum number of loops before animation stops (-1 to play forever)
	int					PlayDelayMs;				// Amount of time to wait before starting animation
	CIwGameCallback		StartedCallback;			// Callback which is called when the animation starts playing
	CIwGameCallback		StoppedCallback;			// Callback which is called when the animation stops playing
	CIwGameCallback		LoopedCallback;				// Callback which is called when the animation loops
public:
	void				setPlaybackSpeed(float speed)	{ PlaybackSpeed = speed; }
	bool				hasFrameChanged() const			{ return FrameHasChanged; }
	void				setPlayDelay(int delay_ms)		{ PlayDelayMs = delay_ms; }
	void				setStartedCallback(CIwGameCallback callback) { StartedCallback = callback; }
	void				setStoppedCallback(CIwGameCallback callback) { StoppedCallback = callback; }
	void				setLoopedCallback(CIwGameCallback callback)	{ LoopedCallback = callback; }
	void				setLooped(int num_loops);
	void				setCurrentFrame(float frame);
	float				getCurrentFrame() const		{ return CurrentFrame; }
	void				Start()						{ Playing = true;  if (PlayDelayMs != 0) PlayDelay.setDuration(PlayDelayMs); }
	void				Stop()						{ Playing = false; }
	bool				isPlaying() const			{ return Playing; }
	void				Restart();
	// Properties End
protected:
	CIwGameTimer		PlayDelay;					// Timer used to time start of animation

public:
	CIwGameAnim() :		Playing(false), CurrentFrame(0), PlaybackSpeed(0), FrameHasChanged(true), Looping(false), NumLoops(0), MaxLoops(0), PlayDelayMs(0), 
						StartedCallback(NULL), StoppedCallback(NULL), LoopedCallback(NULL) {}
	virtual ~CIwGameAnim() {}

	virtual bool		Update(float dt);
};

//
//
// CIwGameAnimImageFrame - Image animation frame
//
//
struct CIwGameAnimImageFrame
{
	short		x, y;		// x,y coordinates of top left hand corner of image on texture atlas
	short		w, h;		// Width and height of image

	CIwGameAnimImageFrame() {}
	CIwGameAnimImageFrame(int _x, int _y, int _w, int _h)
	{
		x = _x;
		y = _y;
		w = _w;
		h = _h;
	}
};

//
//
// CIwGameAnimImage - An image animation
//
//
class CIwGameAnimImage : public CIwGameAnim
{
protected:
	CIwGameAnimImageFrame*	Frames;				// CIwGameAnimFrameManager manages the frames so we do not delete them here
public:
	CIwGameAnimImage() :	CIwGameAnim(), Frames(NULL) {}
	virtual ~CIwGameAnimImage()							{}

	CIwGameAnimImageFrame*	getCurrentFrameData() const { if (CurrentFrame >= 0) return Frames + (int)CurrentFrame; return NULL; }
	void					setFrameData(CIwGameAnimImageFrame* frames, int count) { Frames = frames; FrameCount = count; }
};

//
//
// CIWGameAnimFloat - A floating point variable animation
//
//
class CIWGameAnimFloat  : public CIwGameAnim
{
protected:
	float*					Frames;				// CIwGameAnimFrameManager manages the frames so we do not delete them here

public:
	CIWGameAnimFloat() :	CIwGameAnim(), Frames(NULL) {}
	virtual ~CIWGameAnimFloat()							{}

	void					setFrameData(float* frames, int count) { Frames = frames; FrameCount = count; }
	float					getCurrentFrameData();
};

//
//
// CIwGameAnimFVec2  - A 2D vector variable animation
//
//
class CIwGameAnimFVec2  : public CIwGameAnim
{
protected:
	CIwFVec2*				Frames;				// CIwGameAnimFrameManager manages the frames so we do not delete them here
	CIwFVec2				CurrentData;
public:
	CIwGameAnimFVec2() :	CIwGameAnim(), Frames(NULL) {}
	virtual ~CIwGameAnimFVec2()							{}

	void					setFrameData(CIwFVec2* frames, int count) { Frames = frames; FrameCount = count; }
	const CIwFVec2&			getCurrentFrameData() const { return CurrentData; }
	bool					Update(float dt);
};

//
//
// CIwGameAnimManager - Dual mode animation manager
// Mode 1. UpdateAll = true - Manages the updating of a collection of animation frames (Good if you want to bunch together animations and update them all in one go). This is the default use
// Mode 2. UpdateAll = false - Allow an object to select which animation to play from a selection of animations based on an integer animation index (Good if you want to 
// define the behaviour of an object using a collection of animations and select which one to play at certain points in time). 
// 
// NOTES:
//	No bound checks are performed when setting the current animation
//
// 
class CIwGameAnimManager
{
public:
	// Provide public access to iteration of the animation list
	typedef CIwList<CIwGameAnim*>::iterator	Iterator;
	Iterator						begin() { return Animations.begin(); }
	Iterator						end()	{ return Animations.end(); }
	// Properties
protected:
	CIwList<CIwGameAnim*>			Animations;			// Managed animations
	int								CurrentAnimation;	// Currently playing animation
	bool							UpdateAll;			// Flag to update all anuimations as opposed to updating just the current animation
public:
	void							addAnimation(CIwGameAnim* animation)	{ Animations.push_back(animation); }
	void							setCurrentAnimation(int animation_index);
	CIwGameAnim*					getCurrentAnimation();
	CIwGameAnim*					getAnimation(int animation_index)		{ return Animations.element_at(animation_index); }
	int								getAnimationCount() const				{ return Animations.size(); }
	void							setUpdateAll(bool enable)				{ UpdateAll = enable; }
	// Properties End
protected:

public:
	CIwGameAnimManager() : CurrentAnimation(-1), UpdateAll(true)	{}
	virtual ~CIwGameAnimManager();
	
	virtual void					Update(float dt);	// Updates all animations
};

//
//
// CIwGameAnimFrameManager - Manages the allocation of a collection of animations frames
// TODO: Modify to become pooled
// 
class CIwGameAnimFrameManager
{
	// Properties
protected:
	CIwList<float*>					FloatFrames;		// Floating point animation frames
	CIwList<CIwFVec2*>				FVec2Frames;		// Floating point 2D vector animation frames
	CIwList<CIwGameAnimImageFrame*>	ImageFrames;		// Image animation frames
public:
	float*							allocFloatFrames(int count);
	CIwFVec2*						allocFVec2Frames(int count);
	CIwGameAnimImageFrame*			allocImageFrames(int count);
	CIwGameAnimImageFrame*			allocImageFrames(int count, int frame_w, int frame_h, int start_x, int start_y, int pitch_x, int pitch_y, int image_width);
	float*							getFloatFrames(int index)		{ return FloatFrames.element_at(index); }
	CIwFVec2*						getFVec2Frames(int index)		{ return FVec2Frames.element_at(index); }
	CIwGameAnimImageFrame*			getImageFrames(int index)		{ return ImageFrames.element_at(index); }

	// Properties End
protected:

public:
	CIwGameAnimFrameManager()		{}
	virtual ~CIwGameAnimFrameManager();
};

#endif	// _CIW_GAME_ANIMATION_H_
